@@ -31,15 +31,19 @@ from pikepdf import (
 
 
 class DependencyError(Exception):
-    pass
+    "A third party dependency is needed to extract images of this type."
 
 
 class UnsupportedImageTypeError(Exception):
-    pass
+    "This image is formatted in a way pikepdf does not supported."
 
 
 class NotExtractableError(Exception):
-    pass
+    "Indicates that an image cannot be directly extracted."
+
+
+class InvalidPdfImageError(Exception):
+    "This image is not valid according to the PDF 1.7 specification."
 
 
 def array_str(value):
@@ -69,7 +73,7 @@ def dict_or_array_dict(value):
     if isinstance(value, Dictionary):
         return [value.as_dict()]
     if isinstance(value, Array):
-        return [v.as_list() for v in value]
+        return [v for v in value.as_list()]
     raise NotImplementedError(value)
 
 
@@ -90,7 +94,7 @@ class PdfImageBase(ABC):
 
     @abstractmethod
     def _metadata(self, name, type_, default):
-        pass
+        raise NotImplementedError()
 
     @property
     def width(self):
@@ -155,12 +159,12 @@ class PdfImageBase(ABC):
     @property
     @abstractmethod
     def is_inline(self):
-        pass
+        raise NotImplementedError()
 
     @property
     @abstractmethod
     def icc(self):
-        pass
+        raise NotImplementedError()
 
     @property
     def indexed(self):
@@ -172,6 +176,21 @@ class PdfImageBase(ABC):
         """Size of image as (width, height)"""
         return self.width, self.height
 
+    def _approx_mode_from_icc(self):
+        if self.indexed:
+            icc_profile = self._colorspaces[1][1]
+        else:
+            icc_profile = self._colorspaces[1]
+        icc_profile_nchannels = int(icc_profile['/N'])
+
+        if icc_profile_nchannels == 1:
+            return 'L'
+
+        # Multiple channels, need to open the profile and look
+        mode_from_xcolor_space = {'RGB ': 'RGB', 'CMYK': 'CMYK'}
+        xcolor_space = self.icc.profile.xcolor_space
+        return mode_from_xcolor_space.get(xcolor_space, '')
+
     @property
     def mode(self):
         """``PIL.Image.mode`` equivalent for this image, where possible
@@ -194,18 +213,15 @@ class PdfImageBase(ABC):
                 m = 'CMYK'
             elif self.colorspace == '/ICCBased':
                 try:
-                    icc_profile = self._colorspaces[1]
-                    icc_profile_nchannels = int(icc_profile['/N'])
-                    if icc_profile_nchannels == 1:
-                        m = 'L'
-                    elif icc_profile_nchannels == 3:
-                        m = 'RGB'
-                    elif icc_profile_nchannels == 4:
-                        m = 'CMYK'
-                except (ValueError, TypeError):
-                    pass
+                    m = self._approx_mode_from_icc()
+                except (ValueError, TypeError) as e:
+                    raise NotImplementedError(
+                        "Not sure how to handle PDF image of this type"
+                    ) from e
         if m == '':
-            raise NotImplementedError("Not sure how to handle PDF image of this type")
+            raise NotImplementedError(
+                "Not sure how to handle PDF image of this type"
+            ) from None
         return m
 
     @property
@@ -252,17 +268,12 @@ class PdfImageBase(ABC):
         elif base == '/DeviceCMYK':
             base = 'CMYK'
         elif base == '/ICCBased':
-            if iccobj['/N'] == 3:
-                base = 'RGB'
-            elif iccobj['/N'] == 4:
-                base = 'CMYK'
-            elif iccobj['/N'] == 1:
-                base = 'L'
+            base = self._approx_mode_from_icc()
         return base, lookup
 
     @abstractmethod
     def as_pil_image(self):
-        pass
+        raise NotImplementedError()
 
     @staticmethod
     def _unstack_compression(buffer, filters):
@@ -280,10 +291,7 @@ class PdfImageBase(ABC):
         """
         data = memoryview(buffer)
         while len(filters) > 1 and filters[0] == '/FlateDecode':
-            try:
-                data = decompress(data)
-            except ZlibError as e:
-                raise UnsupportedImageTypeError() from e
+            data = decompress(data)
             filters = filters[1:]
         return data, filters
 
@@ -462,7 +470,10 @@ class PdfImage(PdfImageBase):
         elif self.bits_per_component == 1:
             if self.filters and self.filters[0] == '/JBIG2Decode':
                 if not jbig2.jbig2dec_available():
-                    raise DependencyError("jbig2dec - not installed")
+                    raise DependencyError(
+                        "jbig2dec - not installed or installed version is too old "
+                        "(older than version 0.15)"
+                    )
                 jbig2_globals_obj = self.filter_decodeparms[0][1].get('/JBIG2Globals')
                 im = jbig2.extract_jbig2(self.obj, jbig2_globals_obj)
             else:
@@ -619,7 +630,7 @@ class PdfImage(PdfImageBase):
         elif k > 0:
             ccitt_group = 3  # Group 3 2-D
         else:
-            ccitt_group = 2  # CCITT 1-D
+            ccitt_group = 2  # Group 3 1-D
         black_is_one = self.decode_parms[0].get("/BlackIs1", False)
         # TIFF spec says:
         # use 0 for white_is_zero (=> black is 1)
@@ -683,7 +694,7 @@ class PdfImage(PdfImageBase):
 
         return tiff_header
 
-    def show(self):
+    def show(self):  # pragma: no cover
         """Show the image however PIL wants to"""
         self.as_pil_image().show()
 
@@ -706,10 +717,6 @@ class PdfJpxImage(PdfImage):
         super().__init__(obj)
         self._jpxpil = self.as_pil_image()
 
-    def __eq__(self, other):  # pylint: disable=useless-super-delegation
-        # self._jpxpil is not relevant to equality
-        return super().__eq__(other)
-
     def _extract_direct(self, *, stream):
         data, filters = self._unstack_compression(
             self.obj.get_raw_stream_buffer(), self.filters
@@ -777,6 +784,7 @@ class PdfInlineImage(PdfImageBase):
         b'/CCF': b'/CCITTFaxDecode',
         b'/DCT': b'/DCTDecode',
     }
+    REVERSE_ABBREVS = {v: k for k, v in ABBREVS.items()}
 
     def __init__(self, *, image_data, image_object: tuple):
         """
@@ -793,7 +801,9 @@ class PdfInlineImage(PdfImageBase):
         self._data = image_data
         self._image_object = image_object
 
-        reparse = b' '.join(self._unparse_obj(obj) for obj in image_object)
+        reparse = b' '.join(
+            self._unparse_obj(obj, remap_names=self.ABBREVS) for obj in image_object
+        )
         try:
             reparsed_obj = Object.parse(b'<< ' + reparse + b' >>')
         except PdfError as e:
@@ -802,20 +812,18 @@ class PdfInlineImage(PdfImageBase):
         self.pil = None
 
     def __eq__(self, other):
-        if self.obj == other.obj and (
+        return self.obj == other.obj and (
             self._data._inline_image_raw_bytes()
             == other._data._inline_image_raw_bytes()
-        ):
-            return True
-        return False
+        )
 
     @classmethod
-    def _unparse_obj(cls, obj):
+    def _unparse_obj(cls, obj, remap_names):
         if isinstance(obj, Object):
             if isinstance(obj, Name):
                 name = obj.unparse(resolved=True)
                 assert isinstance(name, bytes)
-                return cls.ABBREVS.get(name, name)
+                return remap_names.get(name, name)
             else:
                 return obj.unparse(resolved=True)
         elif isinstance(obj, bool):
@@ -829,35 +837,40 @@ class PdfInlineImage(PdfImageBase):
         return metadata_from_obj(self.obj, name, type_, default)
 
     def unparse(self):
-        tokens = []
-        tokens.append(b'BI\n')
-        metadata = []
-        for metadata_obj in self._image_object:
-            unparsed = self._unparse_obj(metadata_obj)
-            assert isinstance(unparsed, bytes)
-            metadata.append(unparsed)
-        tokens.append(b' '.join(metadata))
-        tokens.append(b'\nID\n')
-        tokens.append(self._data._inline_image_raw_bytes())
-        tokens.append(b'EI')
-        return b''.join(tokens)
+        def metadata_tokens():
+            for metadata_obj in self._image_object:
+                unparsed = self._unparse_obj(
+                    metadata_obj, remap_names=self.REVERSE_ABBREVS
+                )
+                assert isinstance(unparsed, bytes)
+                yield unparsed
+
+        def inline_image_tokens():
+            yield b'BI\n'
+            yield b' '.join(m for m in metadata_tokens())
+            yield b'\nID\n'
+            yield self._data._inline_image_raw_bytes()
+            yield b'EI'
+
+        return b''.join(inline_image_tokens())
 
     @property
     def is_inline(self):
         return True
 
     @property
-    def icc(self):
-        raise NotImplementedError("Inline images with ICC profiles are not supported")
+    def icc(self):  # pragma: no cover
+        raise InvalidPdfImageError(
+            "Inline images with ICC profiles are not supported in the PDF specification"
+        )
 
     def __repr__(self):
-        mode = '?'
         try:
             mode = self.mode
-        except Exception:
-            pass
+        except NotImplementedError:
+            mode = '?'
         return (
-            f'<pikepdf.PdfInlineImage JPEG2000 image mode={self.mode} '
+            f'<pikepdf.PdfInlineImage image mode={mode} '
             f'size={self.width}x{self.height} at {hex(id(self))}>'
         )
 
